@@ -87,6 +87,7 @@ impl KeyExpr<'static> {
     }
 }
 
+#[zenoh_macros::internal]
 static KEYEXPR_DUMMY: &keyexpr = unsafe { keyexpr::from_str_unchecked("dummy") };
 
 impl<'a> KeyExpr<'a> {
@@ -111,11 +112,6 @@ impl<'a> KeyExpr<'a> {
         Self(KeyExprInner::Borrowed(KEYEXPR_DUMMY))
     }
 
-    #[cfg(not(feature = "internal"))]
-    pub(crate) fn dummy() -> Self {
-        Self(KeyExprInner::Borrowed(KEYEXPR_DUMMY))
-    }
-
     /// Checks if the key expression is the dummy one.
     /// This method is not supposed to be called in user code,
     /// but may be used in language bindings (zenoh-c)
