@@ -41,6 +41,7 @@ use crate::{
         builders::reply::{ReplyBuilder, ReplyBuilderDelete, ReplyBuilderPut, ReplyErrBuilder},
         bytes::ZBytes,
         encoding::Encoding,
+        handlers::CallbackParameter,
         key_expr::KeyExpr,
         sample::{Locality, Sample, SampleKind},
         selector::Selector,
@@ -247,6 +248,14 @@ impl fmt::Display for Query {
     }
 }
 
+impl CallbackParameter for Query {
+    type Message<'a> = Self;
+
+    fn from_message(msg: Self::Message<'_>) -> Self {
+        msg
+    }
+}
+
 #[zenoh_macros::internal]
 pub struct ReplySample<'a> {
     query: &'a Query,
@@ -330,7 +339,7 @@ impl Query {
 }
 pub(crate) struct QueryableState {
     pub(crate) id: Id,
-    pub(crate) key_expr: WireExpr<'static>,
+    pub(crate) key_expr: KeyExpr<'static>,
     pub(crate) complete: bool,
     pub(crate) origin: Locality,
     pub(crate) callback: Callback<Query>,
@@ -351,6 +360,7 @@ pub(crate) struct QueryableInner {
     pub(crate) session: WeakSession,
     pub(crate) id: Id,
     pub(crate) undeclare_on_drop: bool,
+    pub(crate) key_expr: KeyExpr<'static>,
 }
 
 /// A [`Resolvable`] returned when undeclaring a queryable.
@@ -517,6 +527,12 @@ impl<Handler> Queryable<Handler> {
     pub fn set_background(&mut self, background: bool) {
         self.inner.undeclare_on_drop = !background;
     }
+
+    /// Returns the [`KeyExpr`] this queryable responds to.
+    #[inline]
+    pub fn key_expr(&self) -> &KeyExpr<'static> {
+        &self.inner.key_expr
+    }
 }
 
 impl<Handler> Drop for Queryable<Handler> {
