@@ -226,15 +226,13 @@ pub mod session {
 /// The [`Sample`](crate::sample::Sample) structure is the data unit received from [`Subscriber`](crate::pubsub::Subscriber)
 /// or [`Queryable`](crate::query::Queryable) instances. It contains the payload and all the metadata associated with the data.
 pub mod sample {
-    #[zenoh_macros::unstable]
-    pub use crate::api::sample::Locality;
     #[zenoh_macros::unstable]
     pub use crate::api::sample::{SourceInfo, SourceSn};
     pub use crate::api::{
         builders::sample::{
             SampleBuilder, SampleBuilderAny, SampleBuilderDelete, SampleBuilderPut,
         },
-        sample::{Sample, SampleFields, SampleKind},
+        sample::{Locality, Sample, SampleFields, SampleKind},
     };
 }
 
@@ -292,25 +290,21 @@ pub mod query {
 
     #[zenoh_macros::internal]
     pub use crate::api::queryable::ReplySample;
-    #[zenoh_macros::unstable]
-    pub use crate::api::{
-        builders::querier::{QuerierBuilder, QuerierGetBuilder},
-        querier::Querier,
-        query::ReplyKeyExpr,
-        selector::ZenohParameters,
-    };
     pub use crate::api::{
         builders::{
+            querier::{QuerierBuilder, QuerierGetBuilder},
             queryable::QueryableBuilder,
             reply::{ReplyBuilder, ReplyBuilderDelete, ReplyBuilderPut, ReplyErrBuilder},
         },
+        querier::Querier,
         query::{ConsolidationMode, QueryConsolidation, QueryTarget, Reply, ReplyError},
         queryable::{Query, Queryable, QueryableUndeclaration},
         selector::Selector,
     };
+    #[zenoh_macros::unstable]
+    pub use crate::api::{query::ReplyKeyExpr, selector::ZenohParameters};
 }
 
-#[zenoh_macros::unstable]
 pub mod matching {
     pub use crate::api::{
         builders::matching_listener::MatchingListenerBuilder,
@@ -350,6 +344,8 @@ pub mod matching {
 pub mod handlers {
     #[zenoh_macros::internal]
     pub use crate::api::handlers::locked;
+    #[zenoh_macros::internal]
+    pub use crate::api::handlers::CallbackParameter;
     pub use crate::api::handlers::{
         Callback, CallbackDrop, DefaultHandler, FifoChannel, FifoChannelHandler, IntoHandler,
         RingChannel, RingChannelHandler,
@@ -536,35 +532,42 @@ pub mod internal {
 pub mod shm {
     pub use zenoh_shm::api::{
         buffer::{
+            traits::{
+                BufferRelayoutError, OwnedShmBuf, ResideInShm, ShmBuf, ShmBufIntoImmut, ShmBufMut,
+                ShmBufUnsafeMut,
+            },
+            typed::Typed,
             zshm::{zshm, ZShm},
             zshmmut::{zshmmut, ZShmMut},
         },
         cleanup::cleanup_orphaned_shm_segments,
         client::{shm_client::ShmClient, shm_segment::ShmSegment},
         client_storage::{ShmClientStorage, GLOBAL_CLIENT_STORAGE},
-        common::types::{ChunkID, ProtocolID, SegmentID},
+        common::{
+            types::{ChunkID, ProtocolID, PtrInSegment, SegmentID},
+            with_id::WithProtocolID,
+        },
         protocol_implementations::posix::{
             posix_shm_client::PosixShmClient,
-            posix_shm_provider_backend::{
-                LayoutedPosixShmProviderBackendBuilder, PosixShmProviderBackend,
-                PosixShmProviderBackendBuilder,
-            },
-            protocol_id::POSIX_PROTOCOL_ID,
+            posix_shm_provider_backend::{PosixShmProviderBackend, PosixShmProviderBackendBuilder},
         },
         provider::{
             chunk::{AllocatedChunk, ChunkDescriptor},
+            memory_layout::{
+                BufferLayout, BuildLayout, LayoutForType, MemLayout, MemoryLayout, StaticLayout,
+                TryIntoMemoryLayout,
+            },
             shm_provider::{
-                AllocLayout, AllocLayoutSizedBuilder, AllocPolicy, AsyncAllocPolicy, BlockOn,
-                DeallocEldest, DeallocOptimal, DeallocYoungest, Deallocate, Defragment,
-                DynamicProtocolID, ForceDeallocPolicy, GarbageCollect, JustAlloc,
-                LayoutAllocBuilder, ProtocolIDSource, ProviderAllocBuilder, ShmProvider,
-                ShmProviderBuilder, ShmProviderBuilderBackendID, ShmProviderBuilderID,
-                StaticProtocolID,
+                AllocBuilder, AllocLayout, AllocPolicy, AsyncAllocPolicy, BlockOn, DeallocEldest,
+                DeallocOptimal, DeallocYoungest, Deallocate, Defragment, ForceDeallocPolicy,
+                GarbageCollect, JustAlloc, LayoutAllocBuilder, ProviderAllocBuilder, ShmProvider,
+                ShmProviderBuilder,
             },
             shm_provider_backend::ShmProviderBackend,
             types::{
                 AllocAlignment, BufAllocResult, BufLayoutAllocResult, ChunkAllocResult,
-                MemoryLayout, ZAllocError, ZLayoutAllocError, ZLayoutError,
+                TypedBufAllocResult, TypedBufLayoutAllocResult, ZAllocError, ZLayoutAllocError,
+                ZLayoutError,
             },
         },
     };
