@@ -36,14 +36,15 @@ use zenoh_util::Timed;
 
 use super::{
     face::FaceState,
-    resource::{QueryRoute, QueryTargetQablSet, Resource},
+    resource::{QueryTargetQablSet, Resource},
     tables::{NodeId, RoutingExpr, Tables, TablesLock},
 };
-#[cfg(feature = "unstable")]
-use crate::key_expr::KeyExpr;
-use crate::net::routing::{
-    hat::{HatTrait, SendDeclare},
-    router::get_or_set_route,
+use crate::{
+    key_expr::KeyExpr,
+    net::routing::{
+        hat::{HatTrait, SendDeclare},
+        router::{get_or_set_route, QueryRouteBuilder},
+    },
 };
 
 pub(crate) struct Query {
@@ -51,16 +52,14 @@ pub(crate) struct Query {
     src_qid: RequestId,
 }
 
-#[zenoh_macros::unstable]
 #[inline]
 pub(crate) fn get_matching_queryables(
+    hat_code: &(dyn HatTrait + Send + Sync),
     tables: &Tables,
     key_expr: &KeyExpr<'_>,
     complete: bool,
 ) -> HashMap<usize, Arc<FaceState>> {
-    tables
-        .hat_code
-        .get_matching_queryables(tables, key_expr, complete)
+    hat_code.get_matching_queryables(tables, key_expr, complete)
 }
 
 #[allow(clippy::too_many_arguments)]
@@ -101,8 +100,12 @@ pub(crate) fn declare_queryable(
                         .unwrap_or_default();
                     drop(rtables);
                     let mut wtables = zwrite!(tables.tables);
-                    let mut res =
-                        Resource::make_resource(&mut wtables, &mut prefix, expr.suffix.as_ref());
+                    let mut res = Resource::make_resource(
+                        hat_code,
+                        &mut wtables,
+                        &mut prefix,
+                        expr.suffix.as_ref(),
+                    );
                     matches.push(Arc::downgrade(&res));
                     Resource::match_resource(&wtables, &mut res, matches);
                     (res, wtables)
@@ -200,22 +203,20 @@ fn insert_pending_query(outface: &mut Arc<FaceState>, query: Arc<Query>) -> Requ
 
 #[inline]
 fn compute_final_route(
+    hat_code: &(dyn HatTrait + Send + Sync),
     tables: &Tables,
     qabls: &Arc<QueryTargetQablSet>,
     src_face: &Arc<FaceState>,
     expr: &mut RoutingExpr,
     target: &QueryTarget,
     query: Arc<Query>,
-) -> QueryRoute {
+) -> QueryRouteBuilder {
     match target {
         QueryTarget::All => {
-            let mut route = HashMap::new();
+            let mut route = QueryRouteBuilder::new();
             for qabl in qabls.iter() {
-                if tables
-                    .hat_code
-                    .egress_filter(tables, src_face, &qabl.direction.0, expr)
-                {
-                    route.entry(qabl.direction.0.id).or_insert_with(|| {
+                if hat_code.egress_filter(tables, src_face, &qabl.direction.0, expr) {
+                    route.insert(qabl.direction.0.id, || {
                         let mut direction = qabl.direction.clone();
                         let qid = insert_pending_query(&mut direction.0, query.clone());
                         (direction, qid)
@@ -225,14 +226,12 @@ fn compute_final_route(
             route
         }
         QueryTarget::AllComplete => {
-            let mut route = HashMap::new();
+            let mut route = QueryRouteBuilder::new();
             for qabl in qabls.iter() {
                 if qabl.info.map(|info| info.complete).unwrap_or(true)
-                    && tables
-                        .hat_code
-                        .egress_filter(tables, src_face, &qabl.direction.0, expr)
+                    && hat_code.egress_filter(tables, src_face, &qabl.direction.0, expr)
                 {
-                    route.entry(qabl.direction.0.id).or_insert_with(|| {
+                    route.insert(qabl.direction.0.id, || {
                         let mut direction = qabl.direction.clone();
                         let qid = insert_pending_query(&mut direction.0, query.clone());
                         (direction, qid)
@@ -245,15 +244,24 @@ fn compute_final_route(
             if let Some(qabl) = qabls.iter().find(|qabl| {
                 qabl.direction.0.id != src_face.id && qabl.info.is_some_and(|info| info.complete)
             }) {
-                let mut route = HashMap::new();
-
-                let mut direction = qabl.direction.clone();
-                let qid = insert_pending_query(&mut direction.0, query);
-                route.insert(direction.0.id, (direction, qid));
+                let mut route = QueryRouteBuilder::new();
+                route.insert(qabl.direction.0.id, || {
+                    let mut direction = qabl.direction.clone();
+                    let qid = insert_pending_query(&mut direction.0, query);
+                    (direction, qid)
+                });
 
                 route
             } else {
-                compute_final_route(tables, qabls, src_face, expr, &QueryTarget::All, query)
+                compute_final_route(
+                    hat_code,
+                    tables,
+                    qabls,
+                    src_face,
+                    expr,
+                    &QueryTarget::All,
+                    query,
+                )
             }
         }
     }
@@ -356,15 +364,16 @@ pub(crate) fn disable_matches_query_routes(_tables: &mut Tables, res: &mut Arc<R
 
 #[inline]
 fn get_query_route(
+    hat_code: &(dyn HatTrait + Send + Sync),
     tables: &Tables,
     face: &FaceState,
     res: &Option<Arc<Resource>>,
     expr: &mut RoutingExpr,
     routing_context: NodeId,
 ) -> Arc<QueryTargetQablSet> {
-    let hat = &tables.hat_code;
-    let local_context = hat.map_routing_context(tables, face, routing_context);
-    let mut compute_route = || hat.compute_query_route(tables, expr, local_context, face.whatami);
+    let local_context = hat_code.map_routing_context(tables, face, routing_context);
+    let mut compute_route =
+        || hat_code.compute_query_route(tables, expr, local_context, face.whatami);
     if let Some(query_routes) = res
         .as_ref()
         .and_then(|res| res.context.as_ref())
@@ -471,11 +480,20 @@ pub fn route_query(tables_ref: &Arc<TablesLock>, face: &Arc<FaceState>, msg: &mu
                 inc_req_stats!(face, rx, admin, msg.payload)
             }
 
-            if rtables.hat_code.ingress_filter(&rtables, face, &mut expr) {
+            if tables_ref
+                .hat_code
+                .ingress_filter(&rtables, face, &mut expr)
+            {
                 let res = Resource::get_resource(&prefix, expr.suffix);
 
-                let route =
-                    get_query_route(&rtables, face, &res, &mut expr, msg.ext_nodeid.node_id);
+                let route = get_query_route(
+                    tables_ref.hat_code.as_ref(),
+                    &rtables,
+                    face,
+                    &res,
+                    &mut expr,
+                    msg.ext_nodeid.node_id,
+                );
 
                 let query = Arc::new(Query {
                     src_face: face.clone(),
@@ -483,8 +501,16 @@ pub fn route_query(tables_ref: &Arc<TablesLock>, face: &Arc<FaceState>, msg: &mu
                 });
 
                 let queries_lock = zwrite!(tables_ref.queries_lock);
-                let route =
-                    compute_final_route(&rtables, &route, face, &mut expr, &msg.ext_target, query);
+                let route = compute_final_route(
+                    tables_ref.hat_code.as_ref(),
+                    &rtables,
+                    &route,
+                    face,
+                    &mut expr,
+                    &msg.ext_target,
+                    query,
+                )
+                .build();
                 let timeout = msg.ext_timeout.unwrap_or(rtables.queries_default_timeout);
                 drop(queries_lock);
                 drop(rtables);
@@ -503,9 +529,9 @@ pub fn route_query(tables_ref: &Arc<TablesLock>, face: &Arc<FaceState>, msg: &mu
                             ext_tstamp: None,
                         });
                 } else {
-                    for ((outface, key_expr, context), outqid) in route.values() {
+                    for ((outface, key_expr, context), outqid) in route {
                         QueryCleanup::spawn_query_clean_up_task(
-                            outface, tables_ref, *outqid, timeout,
+                            &outface, tables_ref, outqid, timeout,
                         );
                         #[cfg(feature = "stats")]
                         if !admin {
@@ -522,11 +548,11 @@ pub fn route_query(tables_ref: &Arc<TablesLock>, face: &Arc<FaceState>, msg: &mu
                             outqid
                         );
                         outface.primitives.send_request(&mut Request {
-                            id: *outqid,
-                            wire_expr: key_expr.into(),
+                            id: outqid,
+                            wire_expr: key_expr,
                             ext_qos: msg.ext_qos,
                             ext_tstamp: msg.ext_tstamp,
-                            ext_nodeid: ext::NodeIdType { node_id: *context },
+                            ext_nodeid: ext::NodeIdType { node_id: context },
                             ext_target: msg.ext_target,
                             ext_budget: msg.ext_budget,
                             ext_timeout: msg.ext_timeout,
