@@ -14,15 +14,14 @@
 use std::{
     any::Any,
     borrow::{Borrow, Cow},
-    collections::{HashMap, VecDeque},
+    collections::VecDeque,
     convert::TryInto,
     hash::{Hash, Hasher},
     ops::{Deref, DerefMut},
     sync::{Arc, RwLock, Weak},
 };
 
-use ahash::HashMapExt;
-use zenoh_collections::SingleOrBoxHashSet;
+use zenoh_collections::{IntHashMap, IntHashSet, SingleOrBoxHashSet};
 use zenoh_config::WhatAmI;
 use zenoh_protocol::{
     core::{key_expr::keyexpr, ExprId, WireExpr},
@@ -40,7 +39,11 @@ use super::{
     tables::{Tables, TablesLock},
 };
 use crate::net::routing::{
-    dispatcher::face::Face,
+    dispatcher::{
+        face::{Face, FaceId},
+        tables::RoutingExpr,
+    },
+    hat::HatTrait,
     interceptor::{InterceptorTrait, InterceptorsChain},
     router::{disable_matches_data_routes, disable_matches_query_routes},
     RoutingContext,
@@ -49,15 +52,65 @@ use crate::net::routing::{
 pub(crate) type NodeId = u16;
 
 pub(crate) type Direction = (Arc<FaceState>, WireExpr<'static>, NodeId);
-pub(crate) type Route = HashMap<usize, Direction>;
+pub(crate) type Route = Vec<Direction>;
 
-pub(crate) type QueryRoute = HashMap<usize, (Direction, RequestId)>;
 pub(crate) struct QueryTargetQabl {
     pub(crate) direction: Direction,
     pub(crate) info: Option<QueryableInfoType>,
 }
+
+impl QueryTargetQabl {
+    pub(crate) fn new(
+        (fid, ctx): (&FaceId, &Arc<SessionContext>),
+        expr: &mut RoutingExpr,
+        complete: bool,
+    ) -> Option<Self> {
+        let qabl = ctx.qabl?;
+        let key_expr = Resource::get_best_key(expr.prefix, expr.suffix, *fid);
+        Some(Self {
+            direction: (ctx.face.clone(), key_expr.to_owned(), NodeId::default()),
+            info: Some(QueryableInfoType {
+                complete: complete && qabl.complete,
+                // NOTE: local client faces are nearer than remote client faces
+                distance: if ctx.face.is_local { 0 } else { 1 },
+            }),
+        })
+    }
+}
+
 pub(crate) type QueryTargetQablSet = Vec<QueryTargetQabl>;
 
+/// Helper struct to build route, handling face deduplication.
+pub(crate) struct RouteBuilder<T = Direction> {
+    /// The route built.
+    route: Vec<T>,
+    /// The faces' id already inserted.
+    faces: IntHashSet<usize>,
+}
+
+impl<T> RouteBuilder<T> {
+    /// Create a new empty builder.
+    pub(crate) fn new() -> Self {
+        Self {
+            route: Vec::new(),
+            faces: IntHashSet::new(),
+        }
+    }
+
+    /// Insert a new direction if it has not been registered for the given face.
+    pub(crate) fn insert(&mut self, face_id: usize, direction: impl FnOnce() -> T) {
+        if self.faces.insert(face_id) {
+            self.route.push(direction());
+        }
+    }
+
+    /// Build the route, consuming the builder.
+    pub(crate) fn build(self) -> Vec<T> {
+        self.route
+    }
+}
+pub(crate) type QueryRouteBuilder = RouteBuilder<(Direction, RequestId)>;
+
 pub(crate) struct InterceptorCache(Cache<Option<Box<dyn Any + Send + Sync>>>);
 pub(crate) type InterceptorCacheValueType = CacheValueType<Option<Box<dyn Any + Send + Sync>>>;
 
@@ -236,7 +289,7 @@ pub struct Resource {
     pub(crate) nonwild_prefix: Option<Arc<Resource>>,
     pub(crate) children: SingleOrBoxHashSet<Child>,
     pub(crate) context: Option<Box<ResourceContext>>,
-    pub(crate) session_ctxs: ahash::HashMap<usize, Arc<SessionContext>>,
+    pub(crate) session_ctxs: IntHashMap<usize, Arc<SessionContext>>,
 }
 
 impl PartialEq for Resource {
@@ -313,7 +366,7 @@ impl Resource {
             nonwild_prefix,
             children: SingleOrBoxHashSet::new(),
             context: context.map(Box::new),
-            session_ctxs: ahash::HashMap::new(),
+            session_ctxs: IntHashMap::new(),
         }
     }
 
@@ -378,7 +431,7 @@ impl Resource {
             nonwild_prefix: None,
             children: SingleOrBoxHashSet::new(),
             context: None,
-            session_ctxs: ahash::HashMap::new(),
+            session_ctxs: IntHashMap::new(),
         })
     }
 
@@ -432,13 +485,19 @@ impl Resource {
     }
 
     pub fn make_resource(
-        tables: &mut Tables,
+        hat_code: &(dyn HatTrait + Send + Sync),
+        _tables: &mut Tables,
         from: &mut Arc<Resource>,
         mut suffix: &str,
     ) -> Arc<Resource> {
         if !suffix.is_empty() && !suffix.starts_with('/') {
             if let Some(parent) = &mut from.parent.clone() {
-                return Resource::make_resource(tables, parent, &[from.suffix(), suffix].concat());
+                return Resource::make_resource(
+                    hat_code,
+                    _tables,
+                    parent,
+                    &[from.suffix(), suffix].concat(),
+                );
             }
         }
         let mut from = from.clone();
@@ -458,7 +517,7 @@ impl Resource {
             };
             suffix = rest;
         }
-        Resource::upgrade_resource(&mut from, tables.hat_code.new_resource());
+        Resource::upgrade_resource(&mut from, hat_code.new_resource());
         from
     }
 
@@ -807,28 +866,29 @@ pub(crate) fn register_expr(
             }
             None => {
                 let res = Resource::get_resource(&prefix, &expr.suffix);
-                let (mut res, mut wtables) = if res
-                    .as_ref()
-                    .map(|r| r.context.is_some())
-                    .unwrap_or(false)
-                {
-                    drop(rtables);
-                    let wtables = zwrite!(tables.tables);
-                    (res.unwrap(), wtables)
-                } else {
-                    let mut fullexpr = prefix.expr().to_string();
-                    fullexpr.push_str(expr.suffix.as_ref());
-                    let mut matches = keyexpr::new(fullexpr.as_str())
-                        .map(|ke| Resource::get_matches(&rtables, ke))
-                        .unwrap_or_default();
-                    drop(rtables);
-                    let mut wtables = zwrite!(tables.tables);
-                    let mut res =
-                        Resource::make_resource(&mut wtables, &mut prefix, expr.suffix.as_ref());
-                    matches.push(Arc::downgrade(&res));
-                    Resource::match_resource(&wtables, &mut res, matches);
-                    (res, wtables)
-                };
+                let (mut res, mut wtables) =
+                    if res.as_ref().map(|r| r.context.is_some()).unwrap_or(false) {
+                        drop(rtables);
+                        let wtables = zwrite!(tables.tables);
+                        (res.unwrap(), wtables)
+                    } else {
+                        let mut fullexpr = prefix.expr().to_string();
+                        fullexpr.push_str(expr.suffix.as_ref());
+                        let mut matches = keyexpr::new(fullexpr.as_str())
+                            .map(|ke| Resource::get_matches(&rtables, ke))
+                            .unwrap_or_default();
+                        drop(rtables);
+                        let mut wtables = zwrite!(tables.tables);
+                        let mut res = Resource::make_resource(
+                            tables.hat_code.as_ref(),
+                            &mut wtables,
+                            &mut prefix,
+                            expr.suffix.as_ref(),
+                        );
+                        matches.push(Arc::downgrade(&res));
+                        Resource::match_resource(&wtables, &mut res, matches);
+                        (res, wtables)
+                    };
                 let ctx = get_mut_unchecked(&mut res)
                     .session_ctxs
                     .entry(face.id)
@@ -888,8 +948,12 @@ pub(crate) fn register_expr_interest(
                         .unwrap_or_default();
                     drop(rtables);
                     let mut wtables = zwrite!(tables.tables);
-                    let mut res =
-                        Resource::make_resource(&mut wtables, &mut prefix, expr.suffix.as_ref());
+                    let mut res = Resource::make_resource(
+                        tables.hat_code.as_ref(),
+                        &mut wtables,
+                        &mut prefix,
+                        expr.suffix.as_ref(),
+                    );
                     matches.push(Arc::downgrade(&res));
                     Resource::match_resource(&wtables, &mut res, matches);
                     (res, wtables)
