@@ -42,7 +42,6 @@ use zenoh_task::TerminatableTask;
 use zenoh_transport::unicast::TransportUnicast;
 
 use self::{
-    network::Network,
     pubsub::{pubsub_remove_node, undeclare_simple_subscription},
     queries::{queries_remove_node, undeclare_simple_queryable},
 };
@@ -55,7 +54,11 @@ use super::{
 };
 use crate::net::{
     codec::Zenoh080Routing,
-    protocol::linkstate::LinkStateList,
+    protocol::{
+        linkstate::{link_weights_from_config, LinkStateList},
+        network::Network,
+        PEERS_NET_NAME,
+    },
     routing::{
         dispatcher::{face::Face, interests::RemoteInterest},
         hat::TREES_COMPUTATION_DELAY_MS,
@@ -64,7 +67,6 @@ use crate::net::{
 };
 
 mod interests;
-mod network;
 mod pubsub;
 mod queries;
 mod token;
@@ -193,7 +195,7 @@ impl HatBaseTrait for HatCode {
             bail!("\"client\" is not allowed as gossip target")
         }
         let autoconnect = if gossip {
-            AutoConnect::gossip(config, whatami)
+            AutoConnect::gossip(config, whatami, runtime.zid().into())
         } else {
             AutoConnect::disabled()
         };
@@ -202,10 +204,17 @@ impl HatBaseTrait for HatCode {
             unwrap_or_default!(config.routing().peer().mode()) == *"linkstate";
         let router_peers_failover_brokering =
             unwrap_or_default!(config.routing().router().peers_failover_brokering());
+
+        let peer_link_weights = config
+            .routing()
+            .peer()
+            .linkstate()
+            .transport_weights()
+            .clone();
         drop(config_guard);
 
         hat_mut!(tables).linkstatepeers_net = Some(Network::new(
-            "[Peers network]".to_string(),
+            PEERS_NET_NAME.to_string(),
             tables.zid,
             runtime,
             peer_full_linkstate,
@@ -214,6 +223,7 @@ impl HatBaseTrait for HatCode {
             gossip_multihop,
             gossip_target,
             autoconnect,
+            link_weights_from_config(peer_link_weights, PEERS_NET_NAME)?,
         ));
         Ok(())
     }
@@ -428,11 +438,15 @@ impl HatBaseTrait for HatCode {
         face: &FaceState,
         routing_context: NodeId,
     ) -> NodeId {
-        hat!(tables)
-            .linkstatepeers_net
-            .as_ref()
-            .unwrap()
-            .get_local_context(routing_context, face_hat!(face).link_id)
+        if face.whatami != WhatAmI::Client {
+            hat!(tables)
+                .linkstatepeers_net
+                .as_ref()
+                .unwrap()
+                .get_local_context(routing_context, face_hat!(face).link_id)
+        } else {
+            0
+        }
     }
 
     #[inline]
@@ -462,6 +476,40 @@ impl HatBaseTrait for HatCode {
             _ => "graph {}".to_string(),
         }
     }
+
+    fn update_from_config(
+        &self,
+        tables: &mut Tables,
+        tables_ref: &Arc<TablesLock>,
+        runtime: &Runtime,
+    ) -> ZResult<()> {
+        let peer_link_weights = runtime
+            .config()
+            .lock()
+            .0
+            .routing()
+            .peer()
+            .linkstate()
+            .transport_weights()
+            .clone();
+        let peer_link_weights = link_weights_from_config(peer_link_weights, PEERS_NET_NAME)?;
+        if let Some(net) = hat_mut!(tables).linkstatepeers_net.as_mut() {
+            if net.update_link_weights(peer_link_weights) {
+                hat_mut!(tables).schedule_compute_trees(tables_ref.clone());
+            }
+        }
+        Ok(())
+    }
+
+    fn links_info(
+        &self,
+        tables: &Tables,
+    ) -> HashMap<ZenohIdProto, crate::net::protocol::linkstate::LinkInfo> {
+        match &hat!(tables).linkstatepeers_net {
+            Some(net) => net.links_info(),
+            None => HashMap::new(),
+        }
+    }
 }
 
 struct HatContext {
