@@ -31,22 +31,26 @@ use zenoh_protocol::{
 use zenoh_sync::get_mut_unchecked;
 
 use super::{
-    face_hat, face_hat_mut, get_peer, hat, hat_mut, network::Network, push_declaration_profile,
-    res_hat, res_hat_mut, HatCode, HatContext, HatFace, HatTables,
+    face_hat, face_hat_mut, get_peer, hat, hat_mut, push_declaration_profile, res_hat, res_hat_mut,
+    HatCode, HatContext, HatFace, HatTables,
 };
-#[cfg(feature = "unstable")]
-use crate::key_expr::KeyExpr;
-use crate::net::routing::{
-    dispatcher::{
-        face::FaceState,
-        interests::RemoteInterest,
-        pubsub::SubscriberInfo,
-        resource::{NodeId, Resource, SessionContext},
-        tables::{Route, RoutingExpr, Tables},
+use crate::{
+    key_expr::KeyExpr,
+    net::{
+        protocol::network::Network,
+        routing::{
+            dispatcher::{
+                face::FaceState,
+                interests::RemoteInterest,
+                pubsub::SubscriberInfo,
+                resource::{NodeId, Resource, SessionContext},
+                tables::{Route, RoutingExpr, Tables},
+            },
+            hat::{CurrentFutureTrait, HatPubSubTrait, SendDeclare, Sources},
+            router::{disable_matches_data_routes, RouteBuilder},
+            RoutingContext,
+        },
     },
-    hat::{CurrentFutureTrait, HatPubSubTrait, SendDeclare, Sources},
-    router::disable_matches_data_routes,
-    RoutingContext,
 };
 
 #[inline]
@@ -833,21 +837,38 @@ impl HatPubSubTrait for HatCode {
             .linkstatepeer_subs
             .iter()
             .map(|s| {
+                // Compute the list of routers, peers and clients that are known
+                // sources of those subscriptions
+                let mut routers = vec![];
+                let mut peers = vec![];
+                let mut clients = vec![];
+                for s in &res_hat!(s).linkstatepeer_subs {
+                    if let Some(whatami) = hat!(tables)
+                        .linkstatepeers_net
+                        .as_ref()
+                        .and_then(|net| net.get_node(s))
+                        .and_then(|n| n.whatami)
+                    {
+                        match whatami {
+                            WhatAmI::Router => routers.push(*s),
+                            WhatAmI::Peer => peers.push(*s),
+                            WhatAmI::Client => clients.push(*s),
+                        }
+                    } else {
+                        peers.push(*s);
+                    }
+                }
+                for ctx in s.session_ctxs.values().filter(|ctx| {
+                    !ctx.face.is_local && ctx.face.whatami == WhatAmI::Client && ctx.subs.is_some()
+                }) {
+                    clients.push(ctx.face.zid);
+                }
                 (
                     s.clone(),
-                    // Compute the list of routers, peers and clients that are known
-                    // sources of those subscriptions
                     Sources {
-                        routers: vec![],
-                        peers: Vec::from_iter(res_hat!(s).linkstatepeer_subs.iter().cloned()),
-                        clients: s
-                            .session_ctxs
-                            .values()
-                            .filter_map(|f| {
-                                (f.face.whatami == WhatAmI::Client && f.subs.is_some())
-                                    .then_some(f.face.zid)
-                            })
-                            .collect(),
+                        routers,
+                        peers,
+                        clients,
                     },
                 )
             })
@@ -861,7 +882,12 @@ impl HatPubSubTrait for HatCode {
                 if interest.options.subscribers() {
                     if let Some(res) = interest.res.as_ref() {
                         let sources = result.entry(res.clone()).or_insert_with(Sources::default);
-                        match face.whatami {
+                        let whatami = if face.is_local {
+                            tables.whatami
+                        } else {
+                            face.whatami
+                        };
+                        match whatami {
                             WhatAmI::Router => sources.routers.push(face.zid),
                             WhatAmI::Peer => sources.peers.push(face.zid),
                             WhatAmI::Client => sources.clients.push(face.zid),
@@ -882,7 +908,7 @@ impl HatPubSubTrait for HatCode {
     ) -> Arc<Route> {
         #[inline]
         fn insert_faces_for_subs(
-            route: &mut Route,
+            route: &mut RouteBuilder,
             expr: &RoutingExpr,
             tables: &Tables,
             net: &Network,
@@ -898,7 +924,7 @@ impl HatPubSubTrait for HatCode {
                             {
                                 if net.graph.contains_node(direction) {
                                     if let Some(face) = tables.get_face(&net.graph[direction].zid) {
-                                        route.entry(face.id).or_insert_with(|| {
+                                        route.insert(face.id, || {
                                             let key_expr = Resource::get_best_key(
                                                 expr.prefix,
                                                 expr.suffix,
@@ -917,10 +943,10 @@ impl HatPubSubTrait for HatCode {
             }
         }
 
-        let mut route = HashMap::new();
+        let mut route = RouteBuilder::new();
         let key_expr = expr.full_expr();
         if key_expr.ends_with('/') {
-            return Arc::new(route);
+            return Arc::new(route.build());
         }
         tracing::trace!(
             "compute_data_route({}, {:?}, {:?})",
@@ -932,7 +958,7 @@ impl HatPubSubTrait for HatCode {
             Ok(ke) => ke,
             Err(e) => {
                 tracing::warn!("Invalid KE reached the system: {}", e);
-                return Arc::new(route);
+                return Arc::new(route.build());
             }
         };
         let res = Resource::get_resource(expr.prefix, expr.suffix);
@@ -963,7 +989,7 @@ impl HatPubSubTrait for HatCode {
                 if context.subs.is_some()
                     && (source_type == WhatAmI::Client || context.face.whatami == WhatAmI::Client)
                 {
-                    route.entry(*sid).or_insert_with(|| {
+                    route.insert(*sid, || {
                         let key_expr = Resource::get_best_key(expr.prefix, expr.suffix, *sid);
                         (context.face.clone(), key_expr.to_owned(), NodeId::default())
                     });
@@ -971,19 +997,17 @@ impl HatPubSubTrait for HatCode {
             }
         }
         for mcast_group in &tables.mcast_groups {
-            route.insert(
-                mcast_group.id,
+            route.insert(mcast_group.id, || {
                 (
                     mcast_group.clone(),
                     expr.full_expr().to_string().into(),
                     NodeId::default(),
-                ),
-            );
+                )
+            });
         }
-        Arc::new(route)
+        Arc::new(route.build())
     }
 
-    #[zenoh_macros::unstable]
     fn get_matching_subscriptions(
         &self,
         tables: &Tables,
