@@ -37,20 +37,24 @@ use zenoh_protocol::{
 use zenoh_sync::get_mut_unchecked;
 
 use super::{
-    face_hat, face_hat_mut, get_peer, get_router, hat, hat_mut, network::Network,
-    push_declaration_profile, res_hat, res_hat_mut, HatCode, HatContext, HatFace, HatTables,
+    face_hat, face_hat_mut, get_peer, get_router, hat, hat_mut, push_declaration_profile, res_hat,
+    res_hat_mut, HatCode, HatContext, HatFace, HatTables,
 };
-#[cfg(feature = "unstable")]
-use crate::key_expr::KeyExpr;
-use crate::net::routing::{
-    dispatcher::{
-        face::FaceState,
-        resource::{NodeId, Resource, SessionContext},
-        tables::{QueryTargetQabl, QueryTargetQablSet, RoutingExpr, Tables},
+use crate::{
+    key_expr::KeyExpr,
+    net::{
+        protocol::{linkstate::LinkEdgeWeight, network::Network},
+        routing::{
+            dispatcher::{
+                face::FaceState,
+                resource::{NodeId, Resource, SessionContext},
+                tables::{QueryTargetQabl, QueryTargetQablSet, RoutingExpr, Tables},
+            },
+            hat::{CurrentFutureTrait, HatQueriesTrait, SendDeclare, Sources},
+            router::disable_matches_query_routes,
+            RoutingContext,
+        },
     },
-    hat::{CurrentFutureTrait, HatQueriesTrait, SendDeclare, Sources},
-    router::disable_matches_query_routes,
-    RoutingContext,
 };
 
 #[inline]
@@ -946,7 +950,7 @@ pub(super) fn queries_remove_node(
 pub(super) fn queries_linkstate_change(
     tables: &mut Tables,
     zid: &ZenohIdProto,
-    links: &[ZenohIdProto],
+    links: &HashMap<ZenohIdProto, LinkEdgeWeight>,
     send_declare: &mut SendDeclare,
 ) {
     if let Some(mut src_face) = tables.get_face(zid).cloned() {
@@ -964,7 +968,7 @@ pub(super) fn queries_linkstate_change(
                         && !res.session_ctxs.values().any(|ctx| {
                             ctx.face.whatami == WhatAmI::Peer
                                 && src_face.id != ctx.face.id
-                                && HatTables::failover_brokering_to(links, ctx.face.zid)
+                                && HatTables::failover_brokering_to(links, &ctx.face.zid)
                         })
                 })
                 .cloned()
@@ -993,7 +997,7 @@ pub(super) fn queries_linkstate_change(
 
             for mut dst_face in tables.faces.values().cloned() {
                 if src_face.id != dst_face.id
-                    && HatTables::failover_brokering_to(links, dst_face.zid)
+                    && HatTables::failover_brokering_to(links, &dst_face.zid)
                 {
                     for res in face_hat!(src_face).remote_qabls.values() {
                         if !face_hat!(dst_face).local_qabls.contains_key(res) {
@@ -1359,31 +1363,36 @@ impl HatQueriesTrait for HatCode {
             .router_qabls
             .iter()
             .map(|s| {
+                // Compute the list of routers, peers and clients that are known
+                // sources of those queryables
+                let routers = Vec::from_iter(res_hat!(s).router_qabls.keys().cloned());
+                let mut peers = if hat!(tables).full_net(WhatAmI::Peer) {
+                    Vec::from_iter(res_hat!(s).linkstatepeer_qabls.keys().cloned())
+                } else {
+                    vec![]
+                };
+                let mut clients = vec![];
+                for ctx in s
+                    .session_ctxs
+                    .values()
+                    .filter(|ctx| ctx.qabl.is_some() && !ctx.face.is_local)
+                {
+                    match ctx.face.whatami {
+                        WhatAmI::Router => (),
+                        WhatAmI::Peer => {
+                            if !hat!(tables).full_net(WhatAmI::Peer) {
+                                peers.push(ctx.face.zid);
+                            }
+                        }
+                        WhatAmI::Client => clients.push(ctx.face.zid),
+                    }
+                }
                 (
                     s.clone(),
-                    // Compute the list of routers, peers and clients that are known
-                    // sources of those queryables
                     Sources {
-                        routers: Vec::from_iter(res_hat!(s).router_qabls.keys().cloned()),
-                        peers: if hat!(tables).full_net(WhatAmI::Peer) {
-                            Vec::from_iter(res_hat!(s).linkstatepeer_qabls.keys().cloned())
-                        } else {
-                            s.session_ctxs
-                                .values()
-                                .filter_map(|f| {
-                                    (f.face.whatami == WhatAmI::Peer && f.qabl.is_some())
-                                        .then_some(f.face.zid)
-                                })
-                                .collect()
-                        },
-                        clients: s
-                            .session_ctxs
-                            .values()
-                            .filter_map(|f| {
-                                (f.face.whatami == WhatAmI::Client && f.qabl.is_some())
-                                    .then_some(f.face.zid)
-                            })
-                            .collect(),
+                        routers,
+                        peers,
+                        clients,
                     },
                 )
             })
@@ -1397,7 +1406,12 @@ impl HatQueriesTrait for HatCode {
                 if interest.options.queryables() {
                     if let Some(res) = interest.res.as_ref() {
                         let sources = result.entry(res.clone()).or_insert_with(Sources::default);
-                        match face.whatami {
+                        let whatami = if face.is_local {
+                            tables.whatami
+                        } else {
+                            face.whatami
+                        };
+                        match whatami {
                             WhatAmI::Router => sources.routers.push(face.zid),
                             WhatAmI::Peer => sources.peers.push(face.zid),
                             WhatAmI::Client => sources.clients.push(face.zid),
@@ -1483,21 +1497,10 @@ impl HatQueriesTrait for HatCode {
             }
 
             if master || source_type == WhatAmI::Router {
-                for (sid, context) in &mres.session_ctxs {
-                    if context.face.whatami != WhatAmI::Router {
-                        let key_expr = Resource::get_best_key(expr.prefix, expr.suffix, *sid);
-                        if let Some(qabl_info) = context.qabl.as_ref() {
-                            route.push(QueryTargetQabl {
-                                direction: (
-                                    context.face.clone(),
-                                    key_expr.to_owned(),
-                                    NodeId::default(),
-                                ),
-                                info: Some(QueryableInfoType {
-                                    complete: complete && qabl_info.complete,
-                                    distance: 1,
-                                }),
-                            });
+                for face_ctx @ (_, ctx) in &mres.session_ctxs {
+                    if ctx.face.whatami != WhatAmI::Router {
+                        if let Some(qabl) = QueryTargetQabl::new(face_ctx, expr, complete) {
+                            route.push(qabl);
                         }
                     }
                 }
@@ -1507,7 +1510,6 @@ impl HatQueriesTrait for HatCode {
         Arc::new(route)
     }
 
-    #[cfg(feature = "unstable")]
     fn get_matching_queryables(
         &self,
         tables: &Tables,
@@ -1523,7 +1525,6 @@ impl HatQueriesTrait for HatCode {
             key_expr,
             complete
         );
-        crate::net::routing::dispatcher::pubsub::get_matching_subscriptions(tables, key_expr);
         let res = Resource::get_resource(&tables.root_res, key_expr);
         let matches = res
             .as_ref()
@@ -1581,7 +1582,6 @@ impl HatQueriesTrait for HatCode {
     }
 }
 
-#[cfg(feature = "unstable")]
 #[inline]
 fn insert_faces_for_qbls(
     route: &mut HashMap<usize, Arc<FaceState>>,
