@@ -12,7 +12,10 @@
 //   ZettaScale Zenoh Team, <zenoh@zettascale.tech>
 //
 
-use std::sync::{atomic::Ordering, Arc};
+use std::{
+    collections::HashMap,
+    sync::{atomic::Ordering, Arc},
+};
 
 use petgraph::graph::NodeIndex;
 use zenoh_protocol::{
@@ -27,14 +30,17 @@ use zenoh_protocol::{
 use zenoh_sync::get_mut_unchecked;
 
 use super::{
-    face_hat, face_hat_mut, get_peer, get_router, hat, hat_mut, network::Network,
-    push_declaration_profile, res_hat, res_hat_mut, HatCode, HatContext, HatFace, HatTables,
+    face_hat, face_hat_mut, get_peer, get_router, hat, hat_mut, push_declaration_profile, res_hat,
+    res_hat_mut, HatCode, HatContext, HatFace, HatTables,
 };
-use crate::net::routing::{
-    dispatcher::{face::FaceState, interests::RemoteInterest, tables::Tables},
-    hat::{CurrentFutureTrait, HatTokenTrait, SendDeclare},
-    router::{NodeId, Resource, SessionContext},
-    RoutingContext,
+use crate::net::{
+    protocol::{linkstate::LinkEdgeWeight, network::Network},
+    routing::{
+        dispatcher::{face::FaceState, interests::RemoteInterest, tables::Tables},
+        hat::{CurrentFutureTrait, HatTokenTrait, SendDeclare},
+        router::{NodeId, Resource, SessionContext},
+        RoutingContext,
+    },
 };
 
 #[inline]
@@ -887,7 +893,7 @@ pub(super) fn token_tree_change(
 pub(super) fn token_linkstate_change(
     tables: &mut Tables,
     zid: &ZenohIdProto,
-    links: &[ZenohIdProto],
+    links: &HashMap<ZenohIdProto, LinkEdgeWeight>,
     send_declare: &mut SendDeclare,
 ) {
     if let Some(mut src_face) = tables.get_face(zid).cloned() {
@@ -905,7 +911,7 @@ pub(super) fn token_linkstate_change(
                         && !res.session_ctxs.values().any(|ctx| {
                             ctx.face.whatami == WhatAmI::Peer
                                 && src_face.id != ctx.face.id
-                                && HatTables::failover_brokering_to(links, ctx.face.zid)
+                                && HatTables::failover_brokering_to(links, &ctx.face.zid)
                         })
                 })
                 .cloned()
@@ -934,7 +940,7 @@ pub(super) fn token_linkstate_change(
 
             for mut dst_face in tables.faces.values().cloned() {
                 if src_face.id != dst_face.id
-                    && HatTables::failover_brokering_to(links, dst_face.zid)
+                    && HatTables::failover_brokering_to(links, &dst_face.zid)
                 {
                     for res in face_hat!(src_face).remote_tokens.values() {
                         if !face_hat!(dst_face).local_tokens.contains_key(res) {
