@@ -17,17 +17,20 @@ use std::{
     sync::{Arc, Mutex},
 };
 
+use itertools::Itertools;
 use serde_json::json;
 use tracing::{error, trace};
 use zenoh_buffers::buffer::SplitBuffer;
 use zenoh_config::{unwrap_or_default, wrappers::ZenohId, ConfigValidator, WhatAmI};
 use zenoh_core::Wait;
+use zenoh_keyexpr::keyexpr;
+use zenoh_link::Link;
+#[cfg(all(feature = "plugins", feature = "runtime_plugins"))]
+use zenoh_plugin_trait::PluginDiff;
 #[cfg(feature = "plugins")]
-use zenoh_plugin_trait::{PluginControl, PluginDiff, PluginStatus};
-#[cfg(feature = "plugins")]
-use zenoh_protocol::core::key_expr::keyexpr;
+use zenoh_plugin_trait::{PluginControl, PluginStatus};
 use zenoh_protocol::{
-    core::{key_expr::OwnedKeyExpr, ExprId, Reliability, WireExpr, EMPTY_EXPR_ID},
+    core::{key_expr::OwnedKeyExpr, ExprId, Reliability, WireExpr, ZenohIdProto, EMPTY_EXPR_ID},
     network::{
         declare::{queryable::ext::QueryableInfoType, QueryableId},
         ext, Declare, DeclareBody, DeclareQueryable, DeclareSubscriber, Interest, Push, Request,
@@ -36,10 +39,12 @@ use zenoh_protocol::{
     zenoh::{PushBody, RequestBody},
 };
 use zenoh_result::ZResult;
-use zenoh_transport::unicast::TransportUnicast;
+#[cfg(feature = "stats")]
+use zenoh_transport::stats::TransportStats;
+use zenoh_transport::{multicast::TransportMulticast, unicast::TransportUnicast, TransportPeer};
 
 use super::{routing::dispatcher::face::Face, Runtime};
-#[cfg(feature = "plugins")]
+#[cfg(all(feature = "plugins", feature = "runtime_plugins"))]
 use crate::api::plugins::PluginsManager;
 use crate::{
     api::{
@@ -95,7 +100,7 @@ impl ConfigValidator for AdminSpace {
 }
 
 impl AdminSpace {
-    #[cfg(feature = "plugins")]
+    #[cfg(all(feature = "plugins", feature = "runtime_plugins"))]
     fn start_plugin(
         plugin_mgr: &mut PluginsManager,
         config: &zenoh_config::PluginLoad,
@@ -203,6 +208,14 @@ impl AdminSpace {
                 .unwrap(),
             Arc::new(queriers_data),
         );
+        if runtime.state.whatami == WhatAmI::Router {
+            handlers.insert(
+                format!("@/{zid_str}/{whatami_str}/route/successor/**")
+                    .try_into()
+                    .unwrap(),
+                Arc::new(route_successor),
+            );
+        }
 
         #[cfg(feature = "plugins")]
         handlers.insert(
@@ -220,7 +233,7 @@ impl AdminSpace {
             Arc::new(plugins_status),
         );
 
-        #[cfg(feature = "plugins")]
+        #[cfg(all(feature = "plugins", feature = "runtime_plugins"))]
         let mut active_plugins = runtime
             .plugins_manager()
             .started_plugins_iter()
@@ -555,6 +568,21 @@ fn local_data(context: &AdminContext, query: Query) {
 
     let transport_mgr = context.runtime.manager().clone();
 
+    #[cfg(feature = "stats")]
+    let export_stats = query
+        .parameters()
+        .iter()
+        .any(|(k, v)| k == "_stats" && v != "false");
+    #[cfg(feature = "stats")]
+    let insert_stats = |mut json: serde_json::Value, stats: Option<&Arc<TransportStats>>| {
+        if export_stats {
+            json.as_object_mut()
+                .unwrap()
+                .insert("stats".into(), json!(stats.map(|s| s.report())));
+        }
+        json
+    };
+
     // plugins info
     #[cfg(feature = "plugins")]
     let plugins: serde_json::Value = {
@@ -574,42 +602,80 @@ fn local_data(context: &AdminContext, query: Query) {
         .map(|locator| json!(locator.as_str()))
         .collect();
 
+    let links_info = context.runtime.get_links_info();
     // transports info
-    let transport_to_json = |transport: &TransportUnicast| {
-        #[allow(unused_mut)]
-        let mut json = json!({
+    let transport_unicast_to_json = |transport: &TransportUnicast| {
+        let link_to_json = |link: &Link| {
+            json!({
+                "src": link.src.to_string(),
+                "dst": link.dst.to_string()
+            })
+        };
+        #[cfg(not(feature = "stats"))]
+        let links = transport
+            .get_links()
+            .unwrap_or_default()
+            .iter()
+            .map(link_to_json)
+            .collect_vec();
+        #[cfg(feature = "stats")]
+        let links = transport
+            .get_link_stats()
+            .unwrap_or_default()
+            .iter()
+            .map(|(link, stats)| insert_stats(link_to_json(link), Some(stats)))
+            .collect_vec();
+        let json = json!({
             "peer": transport.get_zid().map_or_else(|_| "unknown".to_string(), |p| p.to_string()),
             "whatami": transport.get_whatami().map_or_else(|_| "unknown".to_string(), |p| p.to_string()),
-            "links": transport.get_links().map_or_else(
-                |_| Vec::new(),
-                |links| links.iter().map(|link| link.dst.to_string()).collect()
-            ),
+            "links": links,
+            "weight": transport.get_zid().ok().and_then(|zid| links_info.get(&zid))
         });
         #[cfg(feature = "stats")]
-        {
-            let stats = query
-                .parameters()
-                .iter()
-                .any(|(k, v)| k == "_stats" && v != "false");
-            if stats {
-                json.as_object_mut().unwrap().insert(
-                    "stats".to_string(),
-                    transport
-                        .get_stats()
-                        .map_or_else(|_| json!({}), |p| json!(p.report())),
-                );
-            }
-        }
+        let json = insert_stats(json, transport.get_stats().ok().as_ref());
         json
     };
-    let transports: Vec<serde_json::Value> = zenoh_runtime::ZRuntime::Net
-        .block_in_place(transport_mgr.get_transports_unicast())
-        .iter()
-        .map(transport_to_json)
-        .collect();
+    let transport_multicast_peer_to_json =
+        |transport: &TransportMulticast, mcast_peer: &TransportPeer| {
+            let link_to_json = |link: &Link| {
+                json!({
+                    "src": link.src.to_string(),
+                    "dst": link.dst.to_string()
+                })
+            };
+            let links = mcast_peer.links.iter().map(link_to_json).collect_vec();
+            let json = json!({
+                "peer": mcast_peer.zid.to_string(),
+                "whatami": mcast_peer.whatami.to_string(),
+                "group": transport
+                    .get_link()
+                    .ok()
+                    .and_then(|t| t.group.map(|g| g.to_string()))
+                    .unwrap_or("unknown".to_string()),
 
-    #[allow(unused_mut)]
-    let mut json = json!({
+                "links": links,
+            });
+            #[cfg(feature = "stats")]
+            let json = insert_stats(json, transport.get_stats().ok().as_ref());
+            json
+        };
+    let mut transports: Vec<serde_json::Value> = vec![];
+    zenoh_runtime::ZRuntime::Net.block_in_place(async {
+        for transport in transport_mgr.get_transports_unicast().await {
+            transports.push(transport_unicast_to_json(&transport));
+        }
+        for mcast_transport in transport_mgr.get_transports_multicast().await {
+            if let Ok(peers) = mcast_transport.get_peers() {
+                for mcast_peer in &peers {
+                    transports.push(transport_multicast_peer_to_json(
+                        &mcast_transport,
+                        mcast_peer,
+                    ));
+                }
+            }
+        }
+    });
+    let json = json!({
         "zid": context.runtime.state.zid,
         "version": context.version,
         "metadata": context.runtime.config().lock().0.metadata(),
@@ -617,20 +683,8 @@ fn local_data(context: &AdminContext, query: Query) {
         "sessions": transports,
         "plugins": plugins,
     });
-
     #[cfg(feature = "stats")]
-    {
-        let stats = query
-            .parameters()
-            .iter()
-            .any(|(k, v)| k == "_stats" && v != "false");
-        if stats {
-            json.as_object_mut().unwrap().insert(
-                "stats".to_string(),
-                json!(transport_mgr.get_stats().report()),
-            );
-        }
-    }
+    let json = insert_stats(json, Some(&transport_mgr.get_stats()));
 
     tracing::trace!("AdminSpace router_data: {:?}", json);
     let payload = match serde_json::to_vec(&json) {
@@ -692,10 +746,11 @@ fn routers_linkstate_data(context: &AdminContext, query: Query) {
     .try_into()
     .unwrap();
 
-    let tables = zread!(context.runtime.state.router.tables.tables);
+    let tables = &context.runtime.state.router.tables;
+    let rtables = zread!(tables.tables);
 
     if let Err(e) = query
-        .reply(reply_key, tables.hat_code.info(&tables, WhatAmI::Router))
+        .reply(reply_key, tables.hat_code.info(&rtables, WhatAmI::Router))
         .encoding(Encoding::TEXT_PLAIN)
         .wait()
     {
@@ -711,10 +766,11 @@ fn peers_linkstate_data(context: &AdminContext, query: Query) {
     .try_into()
     .unwrap();
 
-    let tables = zread!(context.runtime.state.router.tables.tables);
+    let tables = &context.runtime.state.router.tables;
+    let rtables = zread!(tables.tables);
 
     if let Err(e) = query
-        .reply(reply_key, tables.hat_code.info(&tables, WhatAmI::Peer))
+        .reply(reply_key, tables.hat_code.info(&rtables, WhatAmI::Peer))
         .encoding(Encoding::TEXT_PLAIN)
         .wait()
     {
@@ -723,8 +779,9 @@ fn peers_linkstate_data(context: &AdminContext, query: Query) {
 }
 
 fn subscribers_data(context: &AdminContext, query: Query) {
-    let tables = zread!(context.runtime.state.router.tables.tables);
-    for sub in tables.hat_code.get_subscriptions(&tables) {
+    let tables = &context.runtime.state.router.tables;
+    let rtables = zread!(tables.tables);
+    for sub in tables.hat_code.get_subscriptions(&rtables) {
         let key = KeyExpr::try_from(format!(
             "@/{}/{}/subscriber/{}",
             context.runtime.state.zid,
@@ -747,8 +804,9 @@ fn subscribers_data(context: &AdminContext, query: Query) {
 }
 
 fn publishers_data(context: &AdminContext, query: Query) {
-    let tables = zread!(context.runtime.state.router.tables.tables);
-    for sub in tables.hat_code.get_publications(&tables) {
+    let tables = &context.runtime.state.router.tables;
+    let rtables = zread!(tables.tables);
+    for sub in tables.hat_code.get_publications(&rtables) {
         let key = KeyExpr::try_from(format!(
             "@/{}/{}/publisher/{}",
             context.runtime.state.zid,
@@ -771,8 +829,9 @@ fn publishers_data(context: &AdminContext, query: Query) {
 }
 
 fn queryables_data(context: &AdminContext, query: Query) {
-    let tables = zread!(context.runtime.state.router.tables.tables);
-    for qabl in tables.hat_code.get_queryables(&tables) {
+    let tables = &context.runtime.state.router.tables;
+    let rtables = zread!(tables.tables);
+    for qabl in tables.hat_code.get_queryables(&rtables) {
         let key = KeyExpr::try_from(format!(
             "@/{}/{}/queryable/{}",
             context.runtime.state.zid,
@@ -795,8 +854,9 @@ fn queryables_data(context: &AdminContext, query: Query) {
 }
 
 fn queriers_data(context: &AdminContext, query: Query) {
-    let tables = zread!(context.runtime.state.router.tables.tables);
-    for sub in tables.hat_code.get_queriers(&tables) {
+    let tables = &context.runtime.state.router.tables;
+    let rtables = zread!(tables.tables);
+    for sub in tables.hat_code.get_queriers(&rtables) {
         let key = KeyExpr::try_from(format!(
             "@/{}/{}/querier/{}",
             context.runtime.state.zid,
@@ -818,6 +878,46 @@ fn queriers_data(context: &AdminContext, query: Query) {
     }
 }
 
+fn route_successor(context: &AdminContext, query: Query) {
+    let reply = |keyexpr: &keyexpr, successor: ZenohIdProto| {
+        if let Err(e) = query
+            .reply(keyexpr, serde_json::to_vec(&json!(successor)).unwrap())
+            .encoding(Encoding::APPLICATION_JSON)
+            .wait()
+        {
+            tracing::error!("Error sending AdminSpace reply: {:?}", e);
+        }
+    };
+    let prefix = format!("@/{}/router/route/successor", context.runtime.zid());
+    let tables = &context.runtime.state.router.tables;
+    let rtables = zread!(tables.tables);
+    // Try to shortcut full successor retrieval if suffix matches 'src/<zid>/dst/<zid>' pattern.
+
+    let suffix = query.key_expr().as_str().strip_prefix(&prefix);
+    if let Some((src, dst)) = suffix.and_then(|s| s.strip_prefix("/src/")?.split_once("/dst/")) {
+        if let (Ok(src_zid), Ok(dst_zid)) = (src.parse(), dst.parse()) {
+            if let Some(successor) = tables.hat_code.route_successor(&rtables, src_zid, dst_zid) {
+                reply(query.key_expr(), successor);
+                return;
+            }
+        }
+    }
+    // Reply with every successor suffix matching the keyexpr.
+    let successors = tables.hat_code.route_successors(&rtables);
+    drop(rtables);
+    for entry in successors.iter() {
+        let keyexpr = KeyExpr::new(format!(
+            "{prefix}/src/{src}/dst/{dst}",
+            src = entry.source,
+            dst = entry.destination
+        ))
+        .unwrap();
+        if query.key_expr().intersects(&keyexpr) {
+            reply(&keyexpr, entry.successor);
+        }
+    }
+}
+
 #[cfg(feature = "plugins")]
 fn plugins_data(context: &AdminContext, query: Query) {
     let guard = context.runtime.plugins_manager();
