@@ -12,7 +12,7 @@
 //   ZettaScale Zenoh Team, <zenoh@zettascale.tech>
 //
 use std::{
-    collections::HashSet,
+    collections::{HashMap, HashSet},
     net::{IpAddr, Ipv6Addr, SocketAddr},
     str::FromStr,
     time::Duration,
@@ -40,7 +40,7 @@ use zenoh_protocol::{
 use zenoh_result::{bail, zerror, ZResult};
 
 use super::{Runtime, RuntimeSession};
-use crate::net::common::AutoConnect;
+use crate::net::{common::AutoConnect, protocol::linkstate::LinkInfo};
 
 const RCV_BUF_SIZE: usize = u16::MAX as usize;
 const SCOUT_INITIAL_PERIOD: Duration = Duration::from_millis(1_000);
@@ -197,7 +197,7 @@ impl Runtime {
                 unwrap_or_default!(guard.scouting().multicast().enabled()),
                 unwrap_or_default!(guard.open().return_conditions().connect_scouted()),
                 *unwrap_or_default!(guard.scouting().multicast().listen().peer()),
-                AutoConnect::multicast(guard, WhatAmI::Peer),
+                AutoConnect::multicast(guard, WhatAmI::Peer, self.zid().into()),
                 unwrap_or_default!(guard.scouting().multicast().address()),
                 unwrap_or_default!(guard.scouting().multicast().interface()),
                 Duration::from_millis(unwrap_or_default!(guard.scouting().delay())),
@@ -245,7 +245,7 @@ impl Runtime {
                     .clone(),
                 unwrap_or_default!(guard.scouting().multicast().enabled()),
                 *unwrap_or_default!(guard.scouting().multicast().listen().router()),
-                AutoConnect::multicast(guard, WhatAmI::Router),
+                AutoConnect::multicast(guard, WhatAmI::Router, self.zid().into()),
                 unwrap_or_default!(guard.scouting().multicast().address()),
                 unwrap_or_default!(guard.scouting().multicast().interface()),
                 Duration::from_millis(unwrap_or_default!(guard.scouting().delay())),
@@ -1141,7 +1141,7 @@ impl Runtime {
         tracing::debug!("Waiting for UDP datagram...");
         loop {
             let (n, peer) = mcast_socket.recv_from(&mut buf).await.unwrap();
-            if local_addrs.iter().any(|addr| *addr == peer) {
+            if local_addrs.contains(&peer) {
                 tracing::trace!("Ignore UDP datagram from own socket");
                 continue;
             }
@@ -1199,16 +1199,11 @@ impl Runtime {
         match session.runtime.whatami() {
             WhatAmI::Client => {
                 let runtime = session.runtime.clone();
-                let cancellation_token = runtime.get_cancellation_token();
-
-                session.runtime.spawn(async move {
+                session.runtime.spawn_abortable(async move {
                     let retry_config = runtime.get_global_connect_retry_config();
                     let mut period = retry_config.period();
                     while runtime.start_client().await.is_err() {
-                        tokio::select! {
-                            _ = tokio::time::sleep(period.next_duration()) => {}
-                            _ = cancellation_token.cancelled() => { break; }
-                        }
+                        tokio::time::sleep(period.next_duration()).await;
                     }
                 });
             }
@@ -1238,4 +1233,21 @@ impl Runtime {
             }
         }
     }
+
+    #[allow(dead_code)]
+    pub(crate) fn update_network(&self) -> ZResult<()> {
+        let router = self.router();
+        let _ctrl_lock = zlock!(router.tables.ctrl_lock);
+        let mut tables = zwrite!(router.tables.tables);
+        router
+            .tables
+            .hat_code
+            .update_from_config(&mut tables, &router.tables, self)
+    }
+
+    pub(crate) fn get_links_info(&self) -> HashMap<ZenohIdProto, LinkInfo> {
+        let router = self.router();
+        let tables = zread!(router.tables.tables);
+        router.tables.hat_code.links_info(&tables)
+    }
 }
