@@ -17,16 +17,14 @@ use std::{
 };
 
 use uhlc::HLC;
-use zenoh_buffers::ZBuf;
 use zenoh_config::Config;
 use zenoh_core::zlock;
 use zenoh_protocol::{
     core::{
-        key_expr::keyexpr, Encoding, ExprId, Reliability, WhatAmI, WireExpr, ZenohIdProto,
-        EMPTY_EXPR_ID,
+        key_expr::keyexpr, ExprId, Reliability, WhatAmI, WireExpr, ZenohIdProto, EMPTY_EXPR_ID,
     },
     network::{ext, Declare, DeclareBody, DeclareKeyExpr, Push},
-    zenoh::{PushBody, Put},
+    zenoh::Put,
 };
 
 use crate::{
@@ -75,7 +73,7 @@ fn base_test() {
     let sub_info = SubscriberInfo;
 
     declare_subscription(
-        zlock!(tables.ctrl_lock).as_ref(),
+        tables.hat_code.as_ref(),
         &tables,
         &mut face.upgrade().unwrap(),
         0,
@@ -197,7 +195,7 @@ fn multisub_test() {
     // --------------
     let sub_info = SubscriberInfo;
     declare_subscription(
-        zlock!(tables.ctrl_lock).as_ref(),
+        tables.hat_code.as_ref(),
         &tables,
         &mut face0.state.clone(),
         0,
@@ -213,7 +211,7 @@ fn multisub_test() {
     assert!(res.upgrade().is_some());
 
     declare_subscription(
-        zlock!(tables.ctrl_lock).as_ref(),
+        tables.hat_code.as_ref(),
         &tables,
         &mut face0.state.clone(),
         1,
@@ -225,7 +223,7 @@ fn multisub_test() {
     assert!(res.upgrade().is_some());
 
     undeclare_subscription(
-        zlock!(tables.ctrl_lock).as_ref(),
+        tables.hat_code.as_ref(),
         &tables,
         &mut face0.state.clone(),
         0,
@@ -236,7 +234,7 @@ fn multisub_test() {
     assert!(res.upgrade().is_some());
 
     undeclare_subscription(
-        zlock!(tables.ctrl_lock).as_ref(),
+        tables.hat_code.as_ref(),
         &tables,
         &mut face0.state.clone(),
         1,
@@ -317,7 +315,7 @@ async fn clean_test() {
     let sub_info = SubscriberInfo;
 
     declare_subscription(
-        zlock!(tables.ctrl_lock).as_ref(),
+        tables.hat_code.as_ref(),
         &tables,
         &mut face0.state.clone(),
         0,
@@ -333,7 +331,7 @@ async fn clean_test() {
     assert!(res2.upgrade().is_some());
 
     declare_subscription(
-        zlock!(tables.ctrl_lock).as_ref(),
+        tables.hat_code.as_ref(),
         &tables,
         &mut face0.state.clone(),
         1,
@@ -350,7 +348,7 @@ async fn clean_test() {
     assert!(res3.upgrade().is_some());
 
     undeclare_subscription(
-        zlock!(tables.ctrl_lock).as_ref(),
+        tables.hat_code.as_ref(),
         &tables,
         &mut face0.state.clone(),
         1,
@@ -366,7 +364,7 @@ async fn clean_test() {
     assert!(res3.upgrade().is_none());
 
     undeclare_subscription(
-        zlock!(tables.ctrl_lock).as_ref(),
+        tables.hat_code.as_ref(),
         &tables,
         &mut face0.state.clone(),
         0,
@@ -386,7 +384,7 @@ async fn clean_test() {
     // --------------
     register_expr(&tables, &mut face0.state.clone(), 2, &"todrop3".into());
     declare_subscription(
-        zlock!(tables.ctrl_lock).as_ref(),
+        tables.hat_code.as_ref(),
         &tables,
         &mut face0.state.clone(),
         2,
@@ -402,7 +400,7 @@ async fn clean_test() {
     assert!(res1.upgrade().is_some());
 
     undeclare_subscription(
-        zlock!(tables.ctrl_lock).as_ref(),
+        tables.hat_code.as_ref(),
         &tables,
         &mut face0.state.clone(),
         2,
@@ -419,7 +417,7 @@ async fn clean_test() {
     register_expr(&tables, &mut face0.state.clone(), 3, &"todrop4".into());
     register_expr(&tables, &mut face0.state.clone(), 4, &"todrop5".into());
     declare_subscription(
-        zlock!(tables.ctrl_lock).as_ref(),
+        tables.hat_code.as_ref(),
         &tables,
         &mut face0.state.clone(),
         3,
@@ -429,7 +427,7 @@ async fn clean_test() {
         &mut |p, m| m.with_mut(|m| p.send_declare(m)),
     );
     declare_subscription(
-        zlock!(tables.ctrl_lock).as_ref(),
+        tables.hat_code.as_ref(),
         &tables,
         &mut face0.state.clone(),
         4,
@@ -508,7 +506,7 @@ impl ClientPrimitives {
     }
 
     #[allow(dead_code)]
-    fn get_last_key(&self) -> Option<WireExpr> {
+    fn get_last_key(&self) -> Option<WireExpr<'_>> {
         self.data.lock().unwrap().as_ref().cloned()
     }
 }
@@ -617,7 +615,7 @@ fn client_test() {
         },
     );
     declare_subscription(
-        zlock!(tables.ctrl_lock).as_ref(),
+        tables.hat_code.as_ref(),
         &tables,
         &mut face0.upgrade().unwrap(),
         0,
@@ -668,7 +666,7 @@ fn client_test() {
         },
     );
     declare_subscription(
-        zlock!(tables.ctrl_lock).as_ref(),
+        tables.hat_code.as_ref(),
         &tables,
         &mut face1.upgrade().unwrap(),
         0,
@@ -719,7 +717,7 @@ fn client_test() {
         },
     );
     declare_subscription(
-        zlock!(tables.ctrl_lock).as_ref(),
+        tables.hat_code.as_ref(),
         &tables,
         &mut face2.upgrade().unwrap(),
         0,
@@ -739,19 +737,7 @@ fn client_test() {
             &face.upgrade().unwrap(),
             &mut Push {
                 wire_expr,
-                ext_qos: ext::QoSType::DEFAULT,
-                ext_tstamp: None,
-                ext_nodeid: ext::NodeIdType { node_id: 0 },
-                payload: PushBody::Put(Put {
-                    timestamp: None,
-                    encoding: Encoding::empty(),
-                    ext_sinfo: None,
-                    #[cfg(feature = "shared-memory")]
-                    ext_shm: None,
-                    ext_unknown: vec![],
-                    payload: ZBuf::empty(),
-                    ext_attachment: None,
-                }),
+                ..Put::default().into()
             },
             Reliability::Reliable,
         );
